import { SupportedStorage } from './types';
/**
 * Returns a localStorage-like object that stores the key-value pairs in
 * memory.
 */
export declare function memoryLocalStorageAdapter(store?: {
    [key: string]: string;
}): SupportedStorage;
//# sourceMappingURL=local-storage.d.ts.map