"use strict";
// from https://github.com/MasterKale/SimpleWebAuthn/blob/master/packages/browser/src/types/index.ts
Object.defineProperty(exports, "__esModule", { value: true });
//# sourceMappingURL=webauthn.dom.js.map